﻿#include "mobilefilewidget.h"

#include <QCheckBox>
#include <QScrollBar>
#include <QDebug>

#include "generatetools.h"

using namespace kdk;

const int BUTTON_WIDTH = 20;
const int BUTTON_HEIGHT = 20;

MobileFileWidget::MobileFileWidget(int text, QWidget *parent) : QWidget(parent)
{
    m_text = text;
    qInfo() << "crumb path" << text;
    initUI();
}

MobileFileWidget::~MobileFileWidget()
{
    if (m_loadAnimation != nullptr) {
        m_loadAnimation->deleteLater();
        m_loadAnimation = nullptr;
    }
    if (m_timeLineView != nullptr) {
        m_timeLineView->deleteLater();
        m_timeLineView = nullptr;
    }
}

void MobileFileWidget::showLoadWin()
{
    m_loadAnimation = new LoadAnimation;
    m_loadAnimation->setHighLight();
    m_stackedWin->addWidget(m_loadAnimation);
    m_stackedWin->setCurrentIndex(1);
}

void MobileFileWidget::setMode(QListView::ViewMode mode)
{
    m_mode = mode;
    switch (m_mode) {
    case QListView::ListMode:
        m_modeButton->setIcon(QIcon::fromTheme("view-list-symbolic"));
        m_modeButton->setToolTip(tr("Icon Mode")); // Icon Mode图标模式
        break;
    case QListView::IconMode:
        m_modeButton->setIcon(QIcon::fromTheme("view-grid-symbolic"));
        m_modeButton->setToolTip(tr("List Mode")); // List Mode列表模式
        break;
    default:
        break;
    }
    m_timeLineView->setMode(m_mode);
    Q_EMIT sigModeChange(m_mode);
}

void MobileFileWidget::initUI()
{
    QVBoxLayout *mainVLayout = new QVBoxLayout();
    mainVLayout->setContentsMargins(0, 24, 0, 0);
    this->setLayout(mainVLayout);

    QHBoxLayout *fileListHLayout = new QHBoxLayout();

    m_copyButtonMobileFileWidget = new LevitationButton(this);
    m_copyButtonMobileFileWidget->setIcon(":/connectinterface/searchrefresh.svg");
    m_copyButtonMobileFileWidget->setText(tr("Download"));
    m_copyButtonMobileFileWidget->hide();

    QFont font;
    font.setBold(true);
    m_selectFileLab = new QLabel(this);
    m_selectFileLab->setText(tr("Select File"));
    m_selectFileLab->setFont(font);
    m_selectFileLab->hide();

    m_retreatButton = new QPushButton(this);
    m_retreatButton->setIcon(QIcon::fromTheme("go-next-rtl-symbolic"));
    m_retreatButton->setFixedSize(BUTTON_WIDTH, BUTTON_HEIGHT);
    m_retreatButton->setFlat(true);
    m_retreatButton->setCursor(Qt::PointingHandCursor);

//    QPushButton *forwardButton = new QPushButton(this);
//    forwardButton->setIcon(QIcon::fromTheme("go-next-symbolic"));
//    forwardButton->setFixedSize(BUTTON_WIDTH, BUTTON_HEIGHT);
//    forwardButton->setFlat(true);
//    forwardButton->setCursor(Qt::PointingHandCursor);

    //面包屑
    m_crumb = new KBreadCrumb(this);
    m_crumb->setFont(font);
    m_crumb->setFlat(false);
    m_crumb->addTab(tr("Mobile file list")); // Mobile file list手机文件列表

    QString currentPath;
    int itemIndex;
    if (m_text == ALL_FILE_LIST_TYPE::QQPICTURE) {
        currentPath = tr("QQ") + "/" + tr("Picture"); // QQ QQ    Picture图片
        itemIndex = ALL_FILE_LIST_TYPE::QQPICTURE;
    } else if (m_text == ALL_FILE_LIST_TYPE::QQVIDEO) {
        currentPath = tr("QQ") + "/" + tr("Video"); // QQ QQ     Video视频
        itemIndex = ALL_FILE_LIST_TYPE::QQVIDEO;
    } else if (m_text == ALL_FILE_LIST_TYPE::QQMUSIC) {
        currentPath = tr("QQ") + "/" + tr("Music"); // QQ QQ     Music音乐
        itemIndex = ALL_FILE_LIST_TYPE::QQMUSIC;
    } else if (m_text == ALL_FILE_LIST_TYPE::QQDOCUMENT) {
        currentPath = tr("QQ") + "/" + tr("Document"); // QQ QQ     Document文档
        itemIndex = ALL_FILE_LIST_TYPE::QQDOCUMENT;
    } else if (m_text == ALL_FILE_LIST_TYPE::WECHATPICTURE) {
        currentPath = tr("WeChat") + "/" + tr("Picture"); // WeChat   Picture图片
        itemIndex = ALL_FILE_LIST_TYPE::WECHATPICTURE;
    } else if (m_text == ALL_FILE_LIST_TYPE::WECHATVIDEO) {
        currentPath = tr("WeChat") + "/" + tr("Video"); // WeChat   Video视频
        itemIndex = ALL_FILE_LIST_TYPE::WECHATVIDEO;
    } else if (m_text == ALL_FILE_LIST_TYPE::WECHATMUSIC) {
        currentPath = tr("WeChat") + "/" + tr("Music"); // WeChat   Music音乐
        itemIndex = ALL_FILE_LIST_TYPE::WECHATMUSIC;
    } else if (m_text == ALL_FILE_LIST_TYPE::WECHATDOCUMENT) {
        currentPath = tr("WeChat") + "/" + tr("Document"); // WeChat   Document文档
        itemIndex = ALL_FILE_LIST_TYPE::WECHATDOCUMENT;
    } else if (m_text == ALL_FILE_LIST_TYPE::PICTURE) {
        currentPath = tr("Picture"); // Picture图片
        itemIndex = ALL_FILE_LIST_TYPE::PICTURE;
    } else if (m_text == ALL_FILE_LIST_TYPE::VIDEO) {
        currentPath = tr("Video"); // Video视频
        itemIndex = ALL_FILE_LIST_TYPE::VIDEO;
    } else if (m_text == ALL_FILE_LIST_TYPE::MUSIC) {
        currentPath = tr("Music"); // Music音乐
        itemIndex = ALL_FILE_LIST_TYPE::MUSIC;
    } else if (m_text == ALL_FILE_LIST_TYPE::DOCUMENT) {
        currentPath = tr("Document"); // Document文档
        itemIndex = ALL_FILE_LIST_TYPE::DOCUMENT;
    } else if (m_text == ALL_FILE_LIST_TYPE::WECHAT) {
        currentPath = tr("WeChat"); // WECHAT微信
        itemIndex = ALL_FILE_LIST_TYPE::WECHAT;
    } else if (m_text == ALL_FILE_LIST_TYPE::QQ) {
        currentPath = tr("QQ"); // QQ QQ
        itemIndex = ALL_FILE_LIST_TYPE::QQ;
    }

    if (currentPath.startsWith(tr("WeChat") + "/") || currentPath.startsWith(tr("QQ") + "/")) { // WeChat微信      QQ QQ
        QStringList list = currentPath.split("/");
        for (int i = 0; i < list.size(); i++) {
            m_crumb->addTab(list.value(i));
        }
        currentPath = list[1];
        m_crumb->setCurrentIndex(list.size());
    } else {
        m_crumb->addTab(currentPath);
        m_crumb->setCurrentIndex(1);
    }

    m_crumb->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

    m_searchLine = new SearchEdit(this);
    m_searchLine->hide();

    m_searchButton = new QPushButton(this);
    m_searchButton->setIcon(QIcon::fromTheme("system-search-symbolic"));
    m_searchButton->setFixedSize(BUTTON_WIDTH, BUTTON_HEIGHT);
    m_searchButton->setFlat(true);
    m_searchButton->setCursor(Qt::PointingHandCursor);

    m_editButton = new QPushButton(this);
    m_editButton->setIcon(QIcon::fromTheme("document-edit-symbolic"));
    m_editButton->setFixedSize(BUTTON_WIDTH, BUTTON_HEIGHT);
    m_editButton->setFlat(true);
    m_editButton->setCursor(Qt::PointingHandCursor);

    m_selectAllLab = new ClickLabel(this);
    m_selectAllLab->setText(tr("Select All"));
    m_selectAllLab->setStyleSheet("color:#3790FA;");
    m_selectAllLab->setCursor(Qt::PointingHandCursor);
    m_selectAllLab->hide();

    m_verticalLineFrame = new QFrame(this);
    m_verticalLineFrame->setFixedHeight(16);
    m_verticalLineFrame->setFrameShape(QFrame::VLine);
    m_verticalLineFrame->setFrameShadow(QFrame::Raised);
    m_verticalLineFrame->setLineWidth(1);

    m_finishLab = new ClickLabel(this);
    m_finishLab->setText(tr("Finish"));
    m_finishLab->setStyleSheet("color:#3790FA;");
    m_finishLab->setCursor(Qt::PointingHandCursor);
    m_finishLab->hide();

    //模式
    m_modeButton = new QPushButton(this);
    m_modeButton->setIcon(QIcon::fromTheme("view-grid-symbolic"));
    m_modeButton->setToolTip(tr("List Mode")); // List Mode列表模式
    m_modeButton->setCursor(Qt::PointingHandCursor);
    m_modeButton->setFixedSize(BUTTON_WIDTH, BUTTON_HEIGHT);
    m_modeButton->setFlat(true);

    //刷新
    m_refreshButton = new QPushButton(this);
    m_refreshButton->setIcon(QIcon::fromTheme("view-refresh-symbolic"));
    m_refreshButton->setToolTip(tr("Refresh")); // Refresh刷新
    m_refreshButton->setCursor(Qt::PointingHandCursor);
    m_refreshButton->setFixedSize(BUTTON_WIDTH, BUTTON_HEIGHT);
    m_refreshButton->setFlat(true);

    QWidget *fileListWidget = new QWidget(this);
    fileListWidget->setLayout(fileListHLayout);
    fileListWidget->setFixedHeight(40);

    if (itemIndex == ALL_FILE_LIST_TYPE::QQMUSIC
        || itemIndex == ALL_FILE_LIST_TYPE::WECHATMUSIC
        || itemIndex == ALL_FILE_LIST_TYPE::MUSIC
        || itemIndex == ALL_FILE_LIST_TYPE::QQDOCUMENT
        || itemIndex == ALL_FILE_LIST_TYPE::WECHATDOCUMENT
        || itemIndex == ALL_FILE_LIST_TYPE::DOCUMENT
            ) {
        m_editButton->hide();
        fileListHLayout->addWidget(m_selectFileLab);
        fileListHLayout->addWidget(m_retreatButton);
    //    fileListHLayout->addWidget(forwardButton);
    //    fileListHLayout->addSpacing(28);
        fileListHLayout->addWidget(m_crumb);
        fileListHLayout->addStretch();
        fileListHLayout->addWidget(m_searchButton);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_editButton);
        fileListHLayout->addWidget(m_selectAllLab);
        fileListHLayout->addWidget(m_verticalLineFrame);
        fileListHLayout->addWidget(m_searchLine);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_finishLab);
        fileListHLayout->addWidget(m_modeButton);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_refreshButton);
        fileListHLayout->setContentsMargins(68, 0, 68, 0);
    } else {
        m_editButton->show();
        fileListHLayout->addWidget(m_selectFileLab);
        fileListHLayout->addWidget(m_retreatButton);
    //    fileListHLayout->addWidget(forwardButton);
    //    fileListHLayout->addSpacing(28);
        fileListHLayout->addWidget(m_crumb);
        fileListHLayout->addStretch();
        fileListHLayout->addWidget(m_searchButton);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_editButton);
        fileListHLayout->addWidget(m_selectAllLab);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_verticalLineFrame);
        fileListHLayout->addWidget(m_searchLine);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_finishLab);
        fileListHLayout->addWidget(m_modeButton);
        fileListHLayout->addSpacing(20);
        fileListHLayout->addWidget(m_refreshButton);
        fileListHLayout->setContentsMargins(68, 0, 68, 0);
    }

    //列表项
    m_timeLineView = new TimeLineView(itemIndex);

    QWidget *listWidget = new QWidget();

    m_scrollArea = new QScrollArea(this);

    QVBoxLayout *scrollVLayout = new QVBoxLayout();
    QWidget *scrollWin = new QWidget(this);
    scrollWin->setLayout(scrollVLayout);
    scrollVLayout->addWidget(m_scrollArea);

    m_scrollArea->setWidgetResizable(true);

    m_scrollArea->setBackgroundRole(QPalette::Base);
    m_scrollArea->setFrameShape(QFrame::NoFrame);
    m_scrollArea->setWidget(listWidget);

    scrollVLayout->setContentsMargins(68, 0, 68, 30);

    QVBoxLayout *layout = new QVBoxLayout();
    listWidget->setLayout(layout);
    layout->addWidget(m_timeLineView);
    layout->setSpacing(0);
    layout->setMargin(0);

    m_stackedWin = new QStackedWidget(this);
    m_stackedWin->addWidget(scrollWin);
    mainVLayout->addWidget(fileListWidget);
    mainVLayout->addSpacing(8);
    mainVLayout->addWidget(m_stackedWin);

    this->setAutoFillBackground(true);
    this->setBackgroundRole(QPalette::Base);

    connect(m_retreatButton, &QPushButton::clicked, this, [=]() {
        int currentIndex = m_crumb->currentIndex();
        if (currentIndex > 0) {
            m_crumb->setCurrentIndex(currentIndex - 1);
        }
    });

    connect(m_searchLine, &SearchEdit::sigEditFocusIn, this, [=]() {
        setEditFocus(true);
    });

    connect(m_searchLine, &SearchEdit::sigEditFocusOut, this, [=]() {
        setEditFocus(false);
    });

    connect(m_searchLine, &SearchEdit::sigSearchResult, this, &MobileFileWidget::sigSearchResult);

    connect(m_searchLine, &SearchEdit::sigSearchInitFail, this, &MobileFileWidget::sigSearchInitFail);

    connect(this, &MobileFileWidget::sigSearchInstalled, m_searchLine, &SearchEdit::onSearchInstalled);

    connect(m_searchButton, &QPushButton::clicked, this, [=]() {
        setEditFocus(true);
    });

    connect(m_editButton, &QPushButton::clicked, this, [=]() {
        setEditButtonState(true);
        m_timeLineView->setDiamond(true);
    });

    connect(m_selectAllLab, &ClickLabel::clicked, this, [=]() {
        if (m_selectAllLab->text() == tr("Select All")) {
            m_selectAllLab->setText(tr("Deselect All"));
            m_timeLineView->selectAll();
        } else if (m_selectAllLab->text() == tr("Deselect All")) {
            m_selectAllLab->setText(tr("Select All"));
            m_timeLineView->deselectAll();
            m_timeLineView->setDiamond(true);
        }
    });

    connect(m_finishLab, &ClickLabel::clicked, this, [=]() {
        setEditButtonState(false);
        m_timeLineView->setDiamond(false);
        m_timeLineView->deselectAll();
    });

    connect(m_refreshButton, &QPushButton::clicked, this, [=]() {
        Q_EMIT sigUpdateClicked(m_text);
    });
    connect(m_modeButton, &QPushButton::clicked, this, &MobileFileWidget::slotModeButtonClicked);
    connect(m_crumb, &KBreadCrumb::currentChanged, this, [=](int index) {
        QString text = m_crumb->tabText(index);
        if (index == 0 && text == tr("Mobile file list")) { // Mobile file list手机文件列表
            slotHideCopyToComputerWidget();
            Q_EMIT sigShowMobileFileListWidget();
            Q_EMIT sigUpdateClicked(ALL_FILE_LIST_TYPE::HOMEPAGE);
            return;
        }

        for (int i = m_crumb->count() - 1; i > index; i--) {
            m_crumb->removeTab(i);
        }

        if (index == 1 && (itemIndex >= 0 && itemIndex < 4)) {
            Q_EMIT sigShowMobileFileSecondWidget(ALL_FILE_LIST_TYPE::QQ);
        } else if (index == 1 && (itemIndex >= 4 && itemIndex < 8)) {
            Q_EMIT sigShowMobileFileSecondWidget(ALL_FILE_LIST_TYPE::WECHAT);
        }
        slotHideCopyToComputerWidget();
    });
    connect(m_timeLineView, &TimeLineView::sigItemInitFinish, this, [=]() {
        m_stackedWin->setCurrentIndex(0);
        Q_EMIT sigInitFinish();
        m_loadAnimation->deleteLater();
        m_loadAnimation = nullptr;
    });

    connect(m_timeLineView, &TimeLineView::sigShowCopyButton, this, &MobileFileWidget::slotShowCopyButtonWidget);
    connect(m_timeLineView, &TimeLineView::sigHideCopyToComputerWidget, this,
            &MobileFileWidget::slotHideCopyToComputerWidget);
    connect(m_timeLineView, &TimeLineView::sigSelectDataList, this, [=](const QList<FileInfo> &fileList) {
        m_selectDataList = fileList;
    });
    connect(m_timeLineView, &TimeLineView::sigSelectAllDataList, this, [=](const QList<FileInfo> &fileList) {
        m_selectDataList = fileList;
    });
    connect(m_timeLineView, &TimeLineView::sigMenuFilePath, this, [=](QString filePath) {
        Q_EMIT sigMenuFilePath(filePath);
    });
    connect(this, &MobileFileWidget::sigSetViewHeight, m_timeLineView, [=]() {
        m_timeLineView->setViewHeight();
    });

    connect(m_copyButtonMobileFileWidget, &QPushButton::clicked, this, [=]() {
        QSettings setting(getenv("HOME") + QString("/.mobilecache/") + QString("setting.ini"), QSettings::IniFormat);
        QString lastPath = setting.value("LastFilePath").toString();
        if (lastPath.isEmpty()) {
            lastPath = getenv("HOME") + QString("/下载");
        }
        QFileDialog fileDialog;
        QString fileName = fileDialog.getExistingDirectory(this, tr("Choose folder"), lastPath); // Choose folder选择文件夹
        if (!fileName.isEmpty()) {
            setting.setValue("LastFilePath", fileName);
            Q_EMIT sigChooseFilePath(fileName, m_selectDataList);
        }
    });
}

void MobileFileWidget::setEditFocus(bool focus)
{
    if (focus == true) {
        m_searchButton->hide();
        m_editButton->hide();
        m_verticalLineFrame->hide();
        m_searchLine->show();
        m_searchLine->setFocus();
    } else {
        m_searchButton->show();
        m_editButton->show();
        m_verticalLineFrame->show();
        m_searchLine->hide();
        m_searchLine->clearFocus();
    }
}

void MobileFileWidget::setEditButtonState(bool state)
{
    if (state == true) {
        m_selectFileLab->show();
        m_retreatButton->hide();
        m_crumb->hide();
        m_searchButton->hide();
        m_editButton->hide();
        m_selectAllLab->show();
        m_verticalLineFrame->show();
        m_searchLine->hide();
        m_finishLab->show();
        m_modeButton->hide();
        m_refreshButton->hide();
    } else {
        m_selectFileLab->hide();
        m_retreatButton->show();
        m_crumb->show();
        m_searchButton->show();
        m_editButton->show();
        m_selectAllLab->hide();
        m_verticalLineFrame->show();
        m_searchLine->hide();
        m_finishLab->hide();
        m_modeButton->show();
        m_refreshButton->show();
    }
}

void MobileFileWidget::slotShowCopyButtonWidget()
{
    if (m_copyButtonMobileFileWidget->isVisible()) {
        moveCopyToComputerWidget();
    } else {
        moveCopyToComputerWidget();
        m_copyButtonMobileFileWidget->show();
        m_copyButtonMobileFileWidget->raise();
    }
}

void MobileFileWidget::slotHideCopyToComputerWidget()
{
    if (m_copyButtonMobileFileWidget->isVisible()) {
        m_copyButtonMobileFileWidget->hide();
    }
}

void MobileFileWidget::moveCopyToComputerWidget()
{
    m_copyButtonMobileFileWidget->move(
        QPoint(this->width() / 2 - m_copyButtonMobileFileWidget->width() / 2,
               this->height() - 16 - m_copyButtonMobileFileWidget->height()));
}

void MobileFileWidget::setThumbanil(QString fileName)
{
    m_timeLineView->setThumbnail(fileName);
}

void MobileFileWidget::slotModeButtonClicked()
{
    int current = 0;
    // 切换视图埋点
    GenerateTools::buriedPoint(kabase::BuriedPoint::PT::KylinMobileAssistantSwitchView);
    switch (m_mode) {
    case QListView::IconMode:
        m_mode = QListView::ListMode;
        m_modeButton->setIcon(QIcon::fromTheme("view-list-symbolic"));
        m_modeButton->setToolTip(tr("Icon Mode")); // Icon Mode图标模式
        current = 1;
        break;
    case QListView::ListMode:
        m_mode = QListView::IconMode;
        m_modeButton->setIcon(QIcon::fromTheme("view-grid-symbolic"));
        m_modeButton->setToolTip(tr("List Mode")); // List Mode列表模式
        current = 0;
        break;
    default:
        break;
    }
    m_timeLineView->setMode(m_mode);
    m_timeLineView->setCurrentViewMode(current);
    Q_EMIT sigModeChange(m_mode);
}

void MobileFileWidget::setTimeLineData(const QList<FileInfo> &listStruct)
{
    m_timeLineView->setTimeLineData(listStruct);
    m_timeLineView->setPictureTimeLineDateWidget();
}

void MobileFileWidget::setMusicData(const QList<FileInfo> &listStruct)
{
    m_timeLineView->setData(listStruct);
    m_timeLineView->setMusicDateWidget();
}

void MobileFileWidget::setDocumenData(const QList<FileInfo> &listStruct)
{
    m_timeLineView->setData(listStruct);
    m_timeLineView->setDocumentDateWidget();
}

void MobileFileWidget::setViewHeight()
{
    Q_EMIT sigSetViewHeight();
    if (m_copyButtonMobileFileWidget) {
        moveCopyToComputerWidget();
    }
}
